require 'json'
require 'rest-client'
require_relative '../conf/conf_api'
require_relative 'handler'



module Recommender
  class Client


    def self.connect(api_key)
      new(api_key)
    end

    def add_item(add_param)
      score = get_action_score(add_param[:engine_name],add_param[:action])
      score != nil ? Handler.load_data(get_load_parameter(add_param,score)) : false
    end

    def load_data(param)
      parameter = {engine_name: param[:engine_name],data: param[:data],api_key: @api_key,token: @token}
      Handler.load_data(parameter)
    end

    def get_similar_users(param)
      parameter = {engine_name: param[:engine_name],api_key: @api_key,token: @token,id: param[:id],nb_wanted: param[:n]}
      Handler.similar_users(parameter)
    end

    def get_similar_items(param)
      parameter = {engine_name: param[:engine_name],api_key: @api_key,token: @token,id: param[:id],nb_wanted: param[:n]}
      Handler.similar_items(parameter)
    end

    def get_recommendations(param)
      parameter = {engine_name: param[:engine_name],api_key: @api_key,token: @token,id: param[:id],nb_wanted: param[:n]}
      Handler.recommend(parameter)
    end

    private

    def initialize(api_key)
      @api_key = api_key
      @token   = Handler.connect_app(get_connect_param(api_key))
    end

    def get_connect_param(api_key)
      {object: {api_key: api_key}.to_json,option: {user: false,app: true}.to_json}
    end

    def get_engine(engine_name)
      param  = {model_name: 'Engine',condition: {name: engine_name}.to_json}
      object = call_api('/restricted/read',param)
      engine = JSON.parse(object)
      engine[0]
    end

    def get_load_parameter(add_param,score)
      {engine_name: add_param[:engine_name],api_key: @api_key,token: @token,data: [{customer_id: add_param[:customer_id],item_id: add_param[:item_id],preference: score}].to_json}

    end

    def get_action_score(engine_name,action)
      score   = nil
      engine  = get_engine(engine_name)
      actions = engine["engine_conf"]["actions"]
      actions.each do |a|
        if a["action"] == action
          score = a["score"].to_i
          break
        end
      end
      score
    end

    def call_api(uri,param)
      begin
        RestClient.get API_SERVER+uri,params: param
      rescue => e
        e.response
      end
    end

  end
end
