require 'json'
require 'redis'
require 'rest-client'
require_relative '../conf/conf_api'

class Handler

   def self.connect_app(app_params)
    waited_keys = [:object,:option]
    call_api(app_params,'/apps/connect','POST',waited_keys)
  end

  def self.load_data(params)
    waited_keys = [:engine_name,:data,:api_key,:token]
    session = get_session_param(params[:api_key],params[:token])
    if valid_reco_parameter(params,waited_keys) && valid_session(session)
      resp = invoke('/apps/load_data','POST',params)
      resp.code == 200 ? true : false
    else
      false
    end
  end

  def self.similar_users(params)
    waited_keys = [:engine_name,:api_key,:id,:nb_wanted,:token]
 session = get_session_param(params[:api_key],params[:token])
    if valid_reco_parameter(params,waited_keys) && valid_parameter_type?(params) && valid_session(session)
      resp = invoke('/users/similar','POST',params)
      resp.code == 200 ? resp.body : false
    else
      false
    end
  end

  def self.similar_items(params)
    waited_keys = [:engine_name,:api_key,:id,:nb_wanted,:token]
 session = get_session_param(params[:api_key],params[:token])
    if valid_reco_parameter(params,waited_keys) && valid_parameter_type?(params) && valid_session(session)
      resp = invoke('/items/similar','POST',params)
      resp.code == 200 ? resp.body : false
    else
      false
    end
  end

  def self.recommend(params)
    waited_keys = [:engine_name,:api_key,:id,:nb_wanted,:token]
 session = get_session_param(params[:api_key],params[:token])
    if valid_reco_parameter(params,waited_keys) && valid_parameter_type?(params) && valid_session(session)
      resp = invoke('/users/recommend','POST',params)
      resp.code == 200 ? resp.body : false
    else
      false
    end
  end

  def self.valid_session(param)
    resp = invoke('/session/valid','GET',param)
    resp.code == 200 ? true : false
  end

  private #######################################

  def self.get_response(request,url,param)
      request[:post] ? response = RestClient.post(url,param) : response = RestClient.get(url, {:params => param})
      response
  end

  def self.get_session_param(api_key,token)
    {object: {api_key: api_key}.to_json,token: token,option: {user: false,app: true}.to_json }
  end

  def self.get_code(response)
    response.code
  end

  def self.get_url(url)
      return API_SERVER+url
  end

  def self.get_request(type)
      request = {post: false,get: false}
      type == 'POST' ? request[:post] = true : request[:get] = true
      request
  end

  def self.invoke(uri,req,params)
    request  = get_request(req)
    url      = get_url(uri)
    response = get_response(request,url,params)
    return response
  end

  def self.valid_parameter?(param)
    param.is_a?(Hash)
  end

  def self.valid_keys?(param,waited_keys)
    waited_keys.each do |k|
      if !(param.has_key?(k))
        return false
        break
     end
    end
    return true
  end

  def self.valid_values?(param)
    parameter = {object: JSON.parse(param[:object],symbolize_names: true),option: JSON.parse(param[:option],symbolize_names: true)}
    values = parameter.values
    values.each do |v|
      if !(v.is_a?(Hash)) || (v.empty?)
        return false
        break
      end
    end
    return true
  end

  def self.valid_option?(option)
    option = JSON.parse(option,symbolize_names: true)
    waited_keys = [:user,:app]
    if valid_keys?(option,waited_keys)
      values = option.values
      values.each do |v|
        unless (v.is_a?(TrueClass) || v.is_a?(FalseClass))
          return false
          break
        end
      end
      if (option[:user] && option[:app]) || (!option[:user] && !option[:app])
        return false
      else
        return true
      end
    else
      return false
    end
  end

  def self.valid_object_keys?(keys,object_keys)
    object_keys.each do |k|
      unless keys.include?(k)
        return false
        break
      end
    end
    return true
  end

  def self.valid_object_values?(values)
    values.each do |v|
      if v.nil?  || !(v.is_a?(String)) || v.empty?
        return false
        break
      end
    end
    return true
  end

  def self.valid_param_object?(object_keys,keys,values)
    response = true
    if !(valid_object_keys?(keys,object_keys)) || !(valid_object_values?(values))
      response = false
    end
    response
  end

  def self.valid_object?(object,option)
    object = JSON.parse(object,symbolize_names: true)
    option = JSON.parse(option,symbolize_names: true)
    keys = object.keys
    values = object.values
    user_keys = [:login,:password]
    app_keys  = [:api_key]
    if option[:user]
      return valid_param_object?(user_keys,keys,values)
    else
      return valid_param_object?(app_keys,keys,values)
    end
  end

  def self.valid_parameter_type?(params)
    params[:engine_name].is_a?(String) && params[:api_key].is_a?(String) && params[:id].is_a?(Fixnum) && params[:nb_wanted].is_a?(Fixnum)
  end

  def self.validate(param,waited_keys)
    valid_parameter?(param) && valid_keys?(param,waited_keys) && valid_values?(param) && valid_option?(param[:option]) && valid_object?(param[:object],param[:option])
  end

  def self.valid_reco_parameter(param,waited_keys)
    valid_parameter?(param) && valid_keys?(param,waited_keys)
  end

  def self.call_api(params,short_uri,rest_req,waited_keys)
   if validate(params,waited_keys)
     response = invoke(short_uri,rest_req,params)
     response == 400 ? false : response
   else
     false
   end
   rescue
     false
  end

end
